###################################################################
###   Adaptive Process Monitoring Using Covariate In formation  ###
###       Real Data Application: Initial Jobless Claims         ###
###################################################################

library(MASS)
library(Matrix)
library(tseries)
library(forecast)

## set your own working directory
setwd('/users/kaiyang/desktop/SPC_Covariates')

## the observed initial jobless claims during 2005-2009
load('IJC_Arizona.rda')  # mydat; 5 yeras, 260 weeks

## Figure 8 in the paper
par(mfrow=c(1,2),mar=c(3,3,1,1),mgp=c(2,1,0))
plot(mydat$time,mydat$claim,type='h',xlab='Time',ylab='Initial jobless claims',xaxt='n',lwd=0.5)
axis(1,at=c(1,53,105,157,209,261),labels=c('2005','2006','2007','2008','2009','2010'))
abline(v=105,lty=3,col='black',lwd=2)
plot(mydat$time,mydat$index,type='h',xlab='Time',ylab='Economic policy uncertainty index',xaxt='n',lwd=0.5)
axis(1,at=c(1,53,105,157,209,261),labels=c('2005','2006','2007','2008','2009','2010'))
abline(v=105,lty=3,col='black',lwd=2)

## the IC dataset: data in years 2005-2006
m <- 104; mydat.ic <- mydat[1:m,]
claim.ic <- mydat.ic$claim
index.ic <- mydat.ic$index

#### fit ARMA models from the IC dataset
fit.claim <- auto.arima(claim.ic,max.p=5,max.q=5,max.d=0,ic='bic')
fit.index <- auto.arima(index.ic,max.p=5,max.q=5,max.d=0,ic='bic')
summary(fit.claim) ## ARMA(1,0): 3429.54; 0.61; 649.47
summary(fit.index) ## ARMA(0,0): 60.02; 20.17

# one-day ahead prediction function for ARMA(1,0) model
Forecast <- function(ts,para){
  l <- length(ts); w <- NULL; est <- NULL
  est[1] <- 0; w[1] <- ts[1]-est[1]
  for(k in 2:l){
    est[k] <- ts[k-1]*para
    w[k] <- ts[k] - est[k]
  }
  result <- list(est,w)
  names(result) <- c("est","residuals")
  return(result)
}

## decorrelated and standardized residuals
n <- 260 # 5 years, 260 weeks
Z.claim <- mydat$claim[1:n]-3429.5
Z.claim <- Forecast(Z.claim,0.61)$residuals
claim.res <- Z.claim/649.47
Z.index <- mydat$index[1:n]-60.02
index.res <- Z.index/20.17
mydat <- data.frame(index.res,claim.res)

## Figure 9 in the paper
par(mfrow=c(1,2),mar=c(3,3,1.5,1),mgp=c(2,1,0))
plot(1:n,mydat$claim.res,type='h',xlab='Time',main='Initial jobless claims',
     ylab='Standardized residual',xaxt='n',lwd=0.7,cex.main=0.8)
axis(1,at=c(1,53,105,157,209,261),labels=c('2005','2006','2007','2008','2009','2010'))
abline(v=105,lty=3,col='black',lwd=2)
plot(1:n,mydat$index.res,type='h',xlab='Time',main='Economic policy uncertainty index',
     ylab='Standardized residual',xaxt='n',lwd=0.7,cex.main=0.8)
axis(1,at=c(1,53,105,157,209,261),labels=c('2005','2006','2007','2008','2009','2010'))
abline(v=105,lty=3,col='black',lwd=2)

## model estimation from the IC data 
index.ic <- index.res[1:m]; claim.ic <- claim.res[1:m] 
summary(lm(claim.ic~index.ic)) ## regression coefficients: -0.020; 0.085

##### bootstrap: control limits
lam <- 0.3; B <- 10000; N <- 1000

### covariate, hx
set.seed(0); Ct <- matrix(0,B,N)
for(b in 1:B) {
  id <- sample(1:m,N,replace=T)
  index.b <- index.ic[id]
  Ct[b,1] <- lam*index.b[1]
  for(i in 2:N) {
    Ct[b,i] <- lam*index.b[i]+(1-lam)*Ct[b,i-1]
  }
}

hl <- 0; hu <- 200
ARL <- rep(0,B); ARL0 <- 200
while(abs(mean(ARL)-ARL0)>0.001) {
  h <- (hl+hu)/2
  for(b in 1:B) {
    id <- which(Ct[b,]>h); id <- c(id,N)
    ARL[b] <- min(id)
  }
  if(mean(ARL)>ARL0) { hu <- h }
  if(mean(ARL)<ARL0) { hl <- h }
  if(abs(hu-hl)<0.001) { break() }
}
hx <- h
# hx=1.115

## quality variable, hy
set.seed(0)
Ct <- Et <- matrix(0,B,N)
for(b in 1:B) {
  id <- sample(1:m,N,replace=T)
  index.b <- index.ic[id]; claim.b <- claim.ic[id]
  Ct[b,1] <- lam*index.b[1]
  w <- ifelse(Ct[b,1]>hx,min(1,lam+Ct[b,1]/hx-1),lam)
  Et[b,1] <- w*claim.b[1]
  for(i in 2:N) {
    Ct[b,i] <- lam*index.b[i]+(1-lam)*Ct[b,i-1]
    w <- ifelse(Ct[b,i]>hx,min(1,lam+Ct[b,i]/hx-1),lam)
    Et[b,i] <- w*claim.b[i]+(1-w)*Et[b,i-1]
  }
}

hl <- 0; hu <- 200
ARL <- rep(0,B); ARL0 <- 200
while(abs(mean(ARL)-ARL0)>0.001) {
  h <- (hl+hu)/2
  for(b in 1:B) {
    id <- which(Et[b,]>h); id <- c(id,N)
    ARL[b] <- min(id)
  }
  if(mean(ARL)>ARL0) { hu <- h }
  if(mean(ARL)<ARL0) { hl <- h }
  if(abs(hu-hl)<0.001) { break() }
}
hy <- h
# hy=1.296

##### sequential monitoring
index.m <- mydat$index.res[(m+1):n]
claim.m <- mydat$claim.res[(m+1):n]
T <- n-m; Ct <- Et <- w <- rep(0,T)
Ct[1] <- lam*index.m[1]
w[1] <- ifelse(Ct[1]>hx,min(1,lam+Ct[1]/hx-1),lam)
Et[1] <- w[1]*claim.m[1]

for(i in 2:T) {
  Ct[i] <- lam*index.m[i]+(1-lam)*Ct[i-1]
  w[i] <- ifelse(Ct[i]>hx,min(1,lam+Ct[i]/hx-1),lam)
  Et[i] <- w[i]*claim.m[i]+(1-w[i])*Et[i-1]
}

## Figure 10 in the paper
par(mfrow=c(1,2),mar=c(3,3.5,1,1),mgp=c(2,1,0))
plot(1:T,Ct,xlab='Time',ylab='Charting statistics for covariate',type='l',lwd=1.3,
     cex.lab=1.4,cex.axis=1.4,xaxt='n')
abline(h=1.115,lty=1,lwd=1.5)
axis(1,at=c(1,53,105,156),labels=c('2007','2008','2009','2010'),
     cex.axis=1.4)
plot(1:T,w,xlab='Time',ylab='Weights',type='l',lwd=1.3,
     cex.lab=1.4,cex.axis=1.4,xaxt='n')
axis(1,at=c(1,53,105,156),labels=c('2007','2008','2009','2010'),
     cex.axis=1.4)

## plot of EWMAC in Figure 11 of the paper
par(mfrow=c(1,1),mar=c(4,4,2,1),mgp=c(2.5,0.8,0))
plot(1:T,Et,xlab='Time',ylab='Charting statistic',type='l',lwd=1.3,
     cex.lab=1.4,cex.axis=1.4,main='EWMAC',cex.main=1.4,xaxt='n')
abline(h=1.296,lty=1,lwd=1.5)
abline(v=54,lty=3,lwd=2.5)
axis(1,at=c(1,53,105,156),labels=c('2007','2008','2009','2010'),
     cex.axis=1.4)

